@@ -30,7 +30,7 @@ class Call():
     >>> Call(["ls", "__no_such_file__"]).check()
     Traceback (most recent call last):
     ...
-    Exception: Call failed with returncode 2: 'ls __no_such_file__ '
+    mini_buildd.HTTPBadRequest: HTTP 400: Internal call failed
     >>> Call(["printf stdin; printf stderr >&2"], stderr=subprocess.STDOUT, shell=True).stdout
     'stdinstderr'
     """
@@ -44,10 +44,7 @@ class Call():
         command line more human-readable and "likely-suitable"
         for cut and paste to a shell.
         """
-        result = ""
-        for arg in call:
-            result += "{arg} ".format(arg="\"" + arg + "\"" if " " in arg else arg)
-        return result
+        return mini_buildd.PyCompat.shlex_join(call)
 
     def __init__(self, call, run_as_root=False, **kwargs):
         self.call = ["sudo", "-n"] + call if run_as_root else call
@@ -64,9 +61,9 @@ class Call():
         self.result = subprocess.run(self.call, **self.kwargs, check=False)
 
         # Convenience 'label' for log output
-        self.label = "{p} {c}..".format(p="#" if run_as_root else "?", c=call[0])
+        self.label = f"{'#' if run_as_root else '?'} {call[0]}.."
 
-        LOG.info("Called with returncode {r}: {c}".format(r=self.result.returncode, c=self._call2shell(self.call)))
+        LOG.info(f"Called with returncode {self.result.returncode}: {self._call2shell(self.call)}")
 
     @classmethod
     def _bos2str(cls, value, errors="replace"):
@@ -92,24 +89,26 @@ class Call():
         """Stderr value (empty string if none)."""
         return self._stdx(self.result.stderr, "stderr")
 
-    def log(self):
+    def log(self, level=None):
         """
         Log calls output to mini-buildd's logging for debugging.
 
-        On error, this logs with level ``warning``. On sucesss,
-        this logs with level ``debug``.
-
+        Per default, this logs to level ``error`` on failure, ``debug`` on success.
         """
-        olog = LOG.debug if self.result.returncode == 0 else LOG.warning
+        level = level if level is not None else logging.DEBUG if self.result.returncode == 0 else logging.ERROR
         for prefix, output in [("stdout", self.stdout), ("stderr", self.stderr)]:
             for line in output.splitlines():
-                olog("{label} ({prefix}): {line}".format(label=self.label, prefix=prefix, line=line.rstrip('\n')))
+                LOG.log(level, f"{self.label} ({prefix}): {line}")
         return self
 
-    def check(self):
+    def success(self):
+        return self.result.returncode == 0
+
+    def check(self, public_message="Internal call failed"):
         """Raise on unsuccessful (returncode != 0) call."""
-        if self.result.returncode != 0:
-            raise Exception("Call failed with returncode {r}: '{c}'".format(r=self.result.returncode, c=self._call2shell(self.call)))
+        if not self.success():
+            self.log()
+            raise mini_buildd.HTTPBadRequest(public_message)
         return self
 
 
@@ -125,21 +124,21 @@ def call_sequence(calls, run_as_root=False, rollback_only=False, **kwargs):
             if calls[i][1]:
                 Call(calls[i][1], run_as_root=run_as_root, **kwargs).log()
             else:
-                LOG.debug("Skipping empty rollback call sequent {i}".format(i=i))
+                LOG.debug(f"Skipping empty rollback call sequent {i}")
 
     if rollback_only:
         rollback(len(calls) - 1)
     else:
         i = 0
         try:
-            for l in calls:
-                if l[0]:
-                    Call(l[0], run_as_root=run_as_root, **kwargs).log().check()
+            for c in calls:
+                if c[0]:
+                    Call(c[0], run_as_root=run_as_root, **kwargs).check()
                 else:
-                    LOG.debug("Skipping empty call sequent {i}".format(i=i))
+                    LOG.debug(f"Skipping empty call sequent {i}")
                 i += 1
         except BaseException:
-            LOG.error("Sequence failed at: {i} (rolling back)".format(i=i))
+            LOG.error(f"Sequence failed at: {i} (rolling back)")
             rollback(i)
             raise
 
@@ -154,16 +153,16 @@ def call_with_retry(call, retry_max_tries=5, retry_sleep=1, retry_failed_cleanup
     >>> call_with_retry(["/bin/false"])
     Traceback (most recent call last):
       ...
-    Exception: Call failed with returncode 1: '/bin/false '
+    mini_buildd.HTTPBadRequest: HTTP 400: Internal call failed
     """
     for t in range(retry_max_tries):
         try:
-            Call(call, **kwargs).log().check()
+            Call(call, **kwargs).check()
             return
         except BaseException as e:
             if t >= retry_max_tries - 1:
                 raise
-            LOG.error("Retrying call in {s} seconds [retry #{t}]: {e}".format(s=retry_sleep, t=t, e=e))
-            if retry_failed_cleanup:
+            LOG.error(f"Retrying call in {retry_sleep} seconds [retry #{t}]: {e}")
+            if retry_failed_cleanup is not None:
                 retry_failed_cleanup()
             time.sleep(retry_sleep)
