@@ -6,7 +6,6 @@ import django
 import django.conf
 
 import mini_buildd.config
-import mini_buildd.models.msglog
 
 LOG = logging.getLogger(__name__)
 
@@ -18,33 +17,8 @@ MBD_INSTALLED_APPS = (
     "django.contrib.sessions",
     "django.contrib.admindocs",
     "django.contrib.staticfiles",
-    "registration",
     "mini_buildd")
 
-MBD_STATIC_CONTEXT = None
-
-
-def context_processor(_request):
-    global MBD_STATIC_CONTEXT  # pylint: disable=global-statement
-    if MBD_STATIC_CONTEXT is None:
-        MBD_STATIC_CONTEXT = {"mbd_version": mini_buildd.__version__}
-        try:
-            from platform import python_version
-            from twisted import __version__ as twisted_version
-            from pyftpdlib import __ver__ as pyftpdlib_version
-            from registration import get_version as registration_version
-
-            MBD_STATIC_CONTEXT["mbd_components"] = {
-                "python": python_version(),
-                "django": django.get_version(),
-                "django-registration": registration_version(),
-                "twisted": twisted_version,
-                "pyftpdlib": pyftpdlib_version}
-        except BaseException:
-            pass
-
-    return MBD_STATIC_CONTEXT
-
 
 class SMTPCreds():
     """
@@ -74,6 +48,11 @@ class SMTPCreds():
         self.port = int(hopo[2])
 
 
+def gen_django_secret_key():
+    # use same randomize-algorithm as in "django/core/management/commands/startproject.py"
+    return "".join([random.choice("abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)") for _i in range(50)])
+
+
 def get_django_secret_key(home):
     """
     Create django's SECRET_KEY *once* and/or returns it.
@@ -89,17 +68,17 @@ def get_django_secret_key(home):
 
     if not os.path.exists(secret_key_filename):
         # use same randomize-algorithm as in "django/core/management/commands/startproject.py"
-        secret_key = "".join([random.choice("abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)") for _i in range(50)])
+        secret_key = gen_django_secret_key()
         with os.fdopen(os.open(secret_key_filename, os.O_CREAT | os.O_WRONLY, 0o600), "w") as secret_key_fd:
             secret_key_fd.write(secret_key)
     else:
-        with open(secret_key_filename, "r") as existing_file:
+        with mini_buildd.fopen(secret_key_filename, "r") as existing_file:
             secret_key = existing_file.read()
 
     return secret_key
 
 
-def configure(smtp_string, loglevel):
+def configure(smtp_string):
     """Configure django."""
     LOG.info("Setting up django...")
 
@@ -108,7 +87,8 @@ def configure(smtp_string, loglevel):
 
     django.conf.settings.configure(
         DEBUG=debug,
-        MESSAGE_LEVEL=mini_buildd.models.msglog.MsgLog.level2django(loglevel),
+
+        DEFAULT_AUTO_FIELD="django.db.models.AutoField",  # django 3.2: https://docs.djangoproject.com/en/3.2/releases/3.2/#customizing-type-of-auto-created-primary-keys
 
         ALLOWED_HOSTS=["*"],
 
@@ -118,48 +98,68 @@ def configure(smtp_string, loglevel):
         EMAIL_HOST_USER=smtp.user,
         EMAIL_HOST_PASSWORD=smtp.password,
 
+        AUTH_PASSWORD_VALIDATORS=[
+            {
+                "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
+            },
+            {
+                "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
+                "OPTIONS": {
+                    "min_length": 8,
+                }
+            },
+            {
+                "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
+            },
+            {
+                "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
+            },
+        ],
+
         DATABASES={"default": {"ENGINE": "django.db.backends.sqlite3",
-                               "NAME": os.path.join(mini_buildd.config.HOME_DIR, "config.sqlite")}},
+                               "NAME": mini_buildd.config.ROUTES["home"].path.join("config.sqlite")}},
 
         TIME_ZONE="Etc/UTC",  # Default time zone: Canonical UTC. Better than django's default 'America/Chicago'.
         USE_TZ=True,          # Django: "stores datetime information in UTC in the database": that's definitely what we want.
         USE_L10N=True,
 
-        SECRET_KEY=get_django_secret_key(mini_buildd.config.HOME_DIR),
+        SECRET_KEY=get_django_secret_key(mini_buildd.config.ROUTES["home"].path.join()),
         ROOT_URLCONF="mini_buildd.urls",
-        STATIC_URL="/static/",
+        STATIC_URL=mini_buildd.config.URIS["static"]["static"].uri,
         ACCOUNT_ACTIVATION_DAYS=3,
-        LOGIN_REDIRECT_URL="/mini_buildd/",
+        LOGIN_URL=mini_buildd.config.URIS["accounts"]["login"].uri,
 
         MIDDLEWARE=[
             "django.middleware.common.CommonMiddleware",
             "django.contrib.sessions.middleware.SessionMiddleware",
             "django.middleware.csrf.CsrfViewMiddleware",
             "django.contrib.auth.middleware.AuthenticationMiddleware",
-            "django.contrib.messages.middleware.MessageMiddleware"],
+            "django.contrib.messages.middleware.MessageMiddleware",
+            "mini_buildd.views.ExceptionMiddleware"
+        ],
 
         INSTALLED_APPS=MBD_INSTALLED_APPS,
         TEMPLATES=[
             {
                 "BACKEND": "django.template.backends.django.DjangoTemplates",
                 "DIRS": [
-                    "{p}/mini_buildd/templates".format(p=mini_buildd.config.PY_PACKAGE_PATH)
+                    f"{mini_buildd.config.PY_PACKAGE_PATH}/mini_buildd/templates"
                 ],
                 "APP_DIRS": True,
                 "OPTIONS": {
                     "debug": debug,
                     "context_processors": [
-                        # former django default (>= 1.9, the default is empty):
-                        "django.contrib.auth.context_processors.auth",
+                        #: Django defaults as per ``django-admin startproject dummy`` (django 2.2.20)
                         "django.template.context_processors.debug",
                         "django.template.context_processors.request",
-                        "django.template.context_processors.i18n",
-                        "django.template.context_processors.media",
-                        "django.template.context_processors.static",
-                        "django.template.context_processors.tz",
+                        "django.contrib.auth.context_processors.auth",
                         "django.contrib.messages.context_processors.messages",
-                        # mini-buildd custom:
-                        "mini_buildd.django_settings.context_processor"
+                        #: Custom
+                        "mini_buildd.views.context",
+                    ],
+                    "builtins": [
+                        "mini_buildd.builtins",
+                        "django.templatetags.static",
                     ],
                 },
             },
@@ -175,6 +175,7 @@ def pseudo_configure():
     Example: Sphinx doc creation, API clients for unpickling model instances.
     """
     django.conf.settings.configure(
+        SECRET_KEY=gen_django_secret_key(),
         DATABASES={
             "default": {
                 "ENGINE": "django.db.backends.sqlite3",
