@@ -1,4 +1,4 @@
-"""
+r"""
 Generic module for models of the django app *mini_buildd*.
 
 Naming conventions
@@ -12,7 +12,7 @@ displays the internal names intelligently to the end user).
 
 *Model class names* must be in **CamelCase**.
 
-*Field names* must be all **lowercase** and **seperatedy by underscores**.
+*Field names* must be all **lowercase** and **separated by underscores**.
 
 For example, **don't** try to do sort of "grouping" using names like::
 
@@ -29,7 +29,7 @@ To group fields together for the end user, use AdminModel's *fieldset* option.
 Methods
 -------
 Any methods that represent mini-buildd logic should go into the models
-directly, but must be prefixed with "mbd_". This avoids conflicts
+directly, but must be prefixed with "mbd\_". This avoids conflicts
 with method names form the django model's class, but still keeps the
 logic where it belongs.
 
@@ -37,38 +37,22 @@ logic where it belongs.
 
 import datetime
 import re
-import pickle
-import base64
 import logging
 
 import django.db.models
-import django.contrib.messages
 import django.contrib.admin
-import django.contrib.auth.models
-import django.db.models.signals
-import django.core.exceptions
+import django.contrib.messages
 import django.template.response
 import django.utils.timezone
+import django.utils.html
 
+import mini_buildd
 import mini_buildd.config
+from mini_buildd.builtins import mbd_cssmap
 
-from mini_buildd.models.msglog import MsgLog
 LOG = logging.getLogger(__name__)
 
 
-# Default action 'delete_selected' action does not call
-# custom delete, nor does it ask prior to deletion.
-#
-# See: https://docs.djangoproject.com/en/dev/ref/contrib/admin/actions/
-#
-# So we just disable this default action. You can still delete
-# single objects from the model's form.
-try:
-    django.contrib.admin.site.disable_action("delete_selected")
-except BaseException as e:
-    LOG.warning("Error disabling delete action (ignoring): {e}".format(e=e))
-
-
 class Model(django.db.models.Model):
     """
     Abstract father model for all mini-buildd models.
@@ -77,22 +61,15 @@ class Model(django.db.models.Model):
     the daemon is running.
     """
 
+    MBD_HELP_EXTRA_OPTIONS = ""  # Set this to non-empty in derived models to "activate"
     extra_options = django.db.models.TextField(blank=True, editable=True,
-                                               help_text="""\
-Extra/experimental options (in the form 'KEY: VALUE' per line) a
-model might support.
+                                               help_text="Extra options (in the form ``<KEY>: <VALUE>``, one per line):")
 
-Note that this is basically just a workaround to easily add
-options to a model without changing the database scheme; i.e.,
-these options may best be described as a staging area, or list
-of 'unofficial features'.
-
-The resp. model documentation should describe what extra options
-are actually supported by the current model.
-""")
-
-    # May be used by any model for persistent python state
-    pickled_data = django.db.models.TextField(blank=True, editable=False)
+    # - 1.0.x: Daemon: Kept last packages as pickled data (base64 encoded).
+    # - 2.0.x: Daemon: Used to save a command line for api call "setup".
+    MBD_HELP_PICKLED_DATA = ""  # Set this to non-empty in derived models to "activate"
+    pickled_data = django.db.models.TextField(blank=True,
+                                              verbose_name="Multi Purpose Field")
 
     class Meta():
         abstract = True
@@ -100,41 +77,70 @@ are actually supported by the current model.
 
     class Admin(django.contrib.admin.ModelAdmin):
         @classmethod
-        def _mbd_on_change(cls, request, obj):
+        def _mbd_on_change(cls, obj):
             """Global actions to take when an object changes."""
             for o in obj.mbd_get_reverse_dependencies():
-                o.mbd_set_changed(request)
-                o.save()
+                if o.LETHAL_DEPENDENCIES:
+                    o.mbd_set_changed()
+                    o.save()
 
         @classmethod
-        def _mbd_on_activation(cls, request, obj):
+        def _mbd_on_activation(cls, obj):
             """Global actions to take when an object becomes active."""
 
         @classmethod
-        def _mbd_on_deactivation(cls, request, obj):
+        def _mbd_on_deactivation(cls, obj):
             """Global actions to take when an object becomes inactive."""
 
         def save_model(self, request, obj, form, change):
             if change:
-                self._mbd_on_change(request, obj)
-
-            obj.save()
+                self._mbd_on_change(obj)
+            super().save_model(request, obj, form, change)
 
         def delete_model(self, request, obj):
-            self._mbd_on_change(request, obj)
+            self._mbd_on_change(obj)
 
             is_prepared_func = getattr(obj, "mbd_is_prepared", None)
             if is_prepared_func and is_prepared_func():
-                self.mbd_remove(request, obj)
+                self.mbd_remove(obj)
 
-            obj.delete()
+            super().delete_model(request, obj)
 
-    @classmethod
-    def mbd_get_daemon(cls):
-        from mini_buildd.daemon import get
-        return get()
+        # All below is only to automatically disable/reorder and get custom help texts for 'pickled_data' and 'extra_options'.
+        def _mbd_help_fields(self):
+            return {"pickled_data": self.model.MBD_HELP_PICKLED_DATA, "extra_options": self.model.MBD_HELP_EXTRA_OPTIONS}
+
+        def get_fields(self, *args, **kwargs):
+            """Make 'extra_options' and 'pickled_data' opt-on and last fields."""
+            fields = super().get_fields(*args, **kwargs)
+
+            for field, help_text in self._mbd_help_fields().items():
+                if field in fields:
+                    fields.remove(field)
+                    if help_text:
+                        fields.append(field)
+
+            return fields
+
+        def __init__(self, *args, **kwargs):
+            """Postprocess fields help_text."""
+            super().__init__(*args, **kwargs)
+
+            help_fields = self._mbd_help_fields()
+            patched = "_mbd_help_text_monkey_patched"
+            for field_obj in self.model._meta.fields:
+                if not hasattr(field_obj, patched):
+                    if field_obj.name in help_fields and help_fields[field_obj.name]:
+                        field_obj.help_text = f"{field_obj.help_text}\n{help_fields[field_obj.name]}"
+                    field_obj.help_text = f"<pre>{django.utils.html.escape(field_obj.help_text)}</pre>"
+                    setattr(field_obj, patched, True)
+
+    def mbd_class_name(self):
+        """Allow to get class name in templates."""
+        return self.__class__.__name__
 
     def mbd_get_extra_options(self):
+        """In case there are multiple entries, the last wins."""
         result = {}
         for line in self.extra_options.splitlines():
             lkey, _lsep, lvalue = line.partition(":")
@@ -145,58 +151,59 @@ are actually supported by the current model.
     def mbd_get_extra_option(self, key, default=None):
         return self.mbd_get_extra_options().get(key, default)
 
-    def mbd_get_pickled_data(self, default=None):
-        try:
-            return pickle.loads(base64.decodebytes(bytes(self.pickled_data, encoding=mini_buildd.config.CHAR_ENCODING)))
-        except BaseException as e:
-            mini_buildd.config.log_exception(LOG, "Ignoring unpickling error", e)
-            return default
-
-    def mbd_set_pickled_data_pickled(self, pickled_data):
-        self.pickled_data = str(base64.encodebytes(pickled_data), encoding=mini_buildd.config.CHAR_ENCODING)
-
-    def mbd_set_pickled_data(self, data):
-        self.mbd_set_pickled_data_pickled(pickle.dumps(data, pickle.HIGHEST_PROTOCOL))
-
     @classmethod
     def mbd_validate_regex(cls, regex, value, field_name):
         if not re.match(regex, value):
-            raise django.core.exceptions.ValidationError("{n} field does not match regex {r}".format(n=field_name, r=regex))
+            raise django.core.exceptions.ValidationError({field_name: f"Field does not match regex {regex}"})
 
-    def mbd_get_dependencies(self):
-        LOG.debug("No dependencies for {s}".format(s=self))
-        return []
-
-    def mbd_get_reverse_dependencies(self):
-        LOG.debug("No reverse dependencies for {s}".format(s=self))
+    @classmethod
+    def mbd_get_dependencies(cls):
         return []
 
     @classmethod
-    def mbd_get_or_create(cls, msglog, **kwargs):
-        """Like get_or_create, but adds a info message."""
-        obj, created = cls.objects.get_or_create(**kwargs)
-        if created:
-            msglog.info("Created: {o}".format(o=obj))
-        else:
-            msglog.debug("Already exists: {o}".format(o=obj))
-        return obj, created
+    def mbd_get_reverse_dependencies(cls):
+        return []
+
+    def save(self, *args, **kwargs):
+        self.full_clean()
+        super().save(*args, **kwargs)
+
+    @classmethod
+    def mbd_diff(cls, obj, **defaults):
+        default = cls(**defaults)
+
+        diff = {}
+        for field_name in defaults:
+            try:
+                current_value = getattr(obj, field_name, None)
+                default_value = getattr(default, field_name, None)
+                if current_value != default_value:
+                    diff[field_name] = {
+                        "current": str(current_value),
+                        "default": str(default_value),
+                    }
+            except BaseException as e:
+                diff[field_name] = {
+                    "not_comparable": f"{field_name}: {e}",
+                }
+        return diff
+
+    @classmethod
+    def mbd_get_default(cls, field_name):
+        return cls._meta.get_field(field_name).get_default()  # pylint: disable=protected-access
 
 
 class StatusModel(Model):
-    """Abstract model class for all models that carry a status. See Manual: :ref:`admin_configuration`."""
+    """Abstract model class for all models that carry a status. See Manual: :ref:`administrator:Models`."""
 
     # The main statuses: removed, prepared, active
     STATUS_REMOVED = 0
     STATUS_PREPARED = 1
     STATUS_ACTIVE = 2
-    STATUS_CHOICES = (
+    STATUS_CHOICES = [
         (STATUS_REMOVED, "Removed"),
         (STATUS_PREPARED, "Prepared"),
-        (STATUS_ACTIVE, "Active"))
-    STATUS_COLORS = {
-        STATUS_REMOVED: {"bg": "red", "fg": "black"},
-        STATUS_PREPARED: {"bg": "yellow", "fg": "black"},
-        STATUS_ACTIVE: {"bg": "green", "fg": "white"}}
+        (STATUS_ACTIVE, "Active")]
     status = django.db.models.IntegerField(choices=STATUS_CHOICES, default=STATUS_REMOVED, editable=False)
 
     # Statuses of the prepared data, relevant for status "Prepared" only.
@@ -224,11 +231,11 @@ class StatusModel(Model):
     class Admin(Model.Admin):
         def save_model(self, request, obj, form, change):
             if change:
-                obj.mbd_set_changed(request)
+                obj.mbd_set_changed()
             super().save_model(request, obj, form, change)
 
         @classmethod
-        def _mbd_run_dependencies(cls, request, obj, func, **kwargs):
+        def _mbd_run_dependencies(cls, obj, func, **kwargs):
             """
             Run action for all dependencies.
 
@@ -239,51 +246,44 @@ class StatusModel(Model):
             """
             for o in obj.mbd_get_dependencies():
                 try:
-                    func(request, o, **kwargs)
+                    func(o, **kwargs)
                 except BaseException as e:
                     if obj.LETHAL_DEPENDENCIES:
                         raise
-                    MsgLog(LOG, request).warning("Check on '{o}' failed: {e}".format(o=o, e=e))
+                    mini_buildd.log_exception(LOG, f"Check on '{o}' failed", e)
 
         @classmethod
-        def mbd_prepare(cls, request, obj):
+        def mbd_prepare(cls, obj):
             if not obj.mbd_is_prepared():
                 # Fresh prepare
-                cls._mbd_run_dependencies(request, obj, cls.mbd_prepare)
-                obj.mbd_prepare(request)
+                cls._mbd_run_dependencies(obj, cls.mbd_prepare)
+                obj.mbd_prepare()
                 obj.status, obj.last_checked = obj.STATUS_PREPARED, obj.CHECK_NONE
                 obj.save()
-                MsgLog(LOG, request).info("Prepared: {o}".format(o=obj))
             elif obj.mbd_is_changed():
                 # Update data on change
-                cls._mbd_run_dependencies(request, obj, cls.mbd_prepare)
-                obj.mbd_sync(request)
+                cls._mbd_run_dependencies(obj, cls.mbd_prepare)
+                obj.mbd_sync()
                 obj.status, obj.last_checked = obj.STATUS_PREPARED, obj.CHECK_NONE
                 obj.save()
-                MsgLog(LOG, request).info("Synced: {o}".format(o=obj))
-            else:
-                MsgLog(LOG, request).info("Already prepared: {o}".format(o=obj))
 
         @classmethod
-        def mbd_check(cls, request, obj, force=False, needs_activation=False):
+        def mbd_check(cls, obj, force=False, needs_activation=False):
             if obj.mbd_is_prepared() and not obj.mbd_is_changed():
                 try:
                     # Also run for all status dependencies
-                    cls._mbd_run_dependencies(request, obj, cls.mbd_check,
+                    cls._mbd_run_dependencies(obj, cls.mbd_check,
                                               force=force,
                                               needs_activation=obj.mbd_is_active() or obj.last_checked == obj.CHECK_REACTIVATE)
 
-                    if force or obj.mbd_needs_check():
-                        last_checked = obj.last_checked
-
-                        obj.mbd_check(request)
+                    if force or not obj.mbd_is_checked():
+                        obj.mbd_check()
 
                         # Handle special flags
                         reactivated = False
                         if obj.last_checked == obj.CHECK_REACTIVATE:
                             obj.status = StatusModel.STATUS_ACTIVE
                             reactivated = True
-                            MsgLog(LOG, request).info("Auto-reactivated: {o}".format(o=obj))
 
                         # Finish up
                         obj.last_checked = django.utils.timezone.now()
@@ -291,60 +291,50 @@ class StatusModel(Model):
 
                         # Run activation hook if reactivated
                         if reactivated:
-                            cls._mbd_on_activation(request, obj)
-
-                        MsgLog(LOG, request).info("Checked ({f}, last={c}): {o}".format(f="forced" if force else "scheduled", c=last_checked, o=obj))
-                    else:
-                        MsgLog(LOG, request).info("Needs no check: {o}".format(o=obj))
+                            cls._mbd_on_activation(obj)
 
                     if needs_activation and not obj.mbd_is_active():
-                        raise Exception("Not active, but a (tobe-)active item depends on it. Activate this first: {o}".format(o=obj))
+                        raise mini_buildd.HTTPBadRequest(f"Not active, but a (tobe-)active item depends on it. Activate this first: {obj}")
                 except BaseException:
                     # Check failed, auto-deactivate and re-raise exception
                     obj.last_checked = max(obj.last_checked, obj.CHECK_FAILED)
                     if obj.mbd_is_active():
                         obj.status, obj.last_checked = obj.STATUS_PREPARED, obj.CHECK_REACTIVATE
-                        MsgLog(LOG, request).error("Automatically deactivated: {o}".format(o=obj))
                     obj.save()
                     raise
             else:
-                raise Exception("Can't check removed or changed object (run 'prepare' first): {o}".format(o=obj))
+                raise mini_buildd.HTTPBadRequest(f"Can't check removed or changed object (run 'prepare' first): {obj}")
 
         @classmethod
-        def mbd_activate(cls, request, obj):
-            if obj.mbd_is_prepared() and obj.mbd_is_checked():
-                cls._mbd_run_dependencies(request, obj, cls.mbd_activate)
-                obj.status = obj.STATUS_ACTIVE
-                obj.save()
-                cls._mbd_on_activation(request, obj)
-                MsgLog(LOG, request).info("Activated: {o}".format(o=obj))
-            elif obj.mbd_is_prepared() and (obj.last_checked == obj.CHECK_FAILED or obj.last_checked == obj.CHECK_NONE):
-                obj.last_checked = obj.CHECK_REACTIVATE
-                obj.save()
-                MsgLog(LOG, request).info("Will auto-activate when check succeeds: {o}".format(o=obj))
-            elif obj.mbd_is_active():
-                MsgLog(LOG, request).info("Already active: {o}".format(o=obj))
-            else:
-                raise Exception("Prepare and check first: {o}".format(o=obj))
+        def mbd_activate(cls, obj):
+            if not obj.mbd_is_active():
+                if obj.mbd_is_prepared() and obj.mbd_is_checked():
+                    cls._mbd_run_dependencies(obj, cls.mbd_activate)
+                    obj.status = obj.STATUS_ACTIVE
+                    obj.save()
+                elif obj.mbd_is_prepared() and obj.last_checked in [obj.CHECK_FAILED, obj.CHECK_NONE]:
+                    obj.last_checked = obj.CHECK_REACTIVATE
+                    obj.save()
+                else:
+                    raise mini_buildd.HTTPBadRequest(f"Prepare and check first: {obj}")
+
+            if obj.mbd_is_active():
+                cls._mbd_on_activation(obj)
 
         @classmethod
-        def mbd_deactivate(cls, request, obj):
+        def mbd_deactivate(cls, obj):
             obj.status = min(obj.STATUS_PREPARED, obj.status)
             if obj.last_checked == obj.CHECK_REACTIVATE:
                 obj.last_checked = obj.CHECK_FAILED
             obj.save()
-            cls._mbd_on_deactivation(request, obj)
-            MsgLog(LOG, request).info("Deactivated: {o}".format(o=obj))
+            cls._mbd_on_deactivation(obj)
 
         @classmethod
-        def mbd_remove(cls, request, obj):
+        def mbd_remove(cls, obj):
             if obj.mbd_is_prepared():
-                obj.mbd_remove(request)
+                obj.mbd_remove()
                 obj.status, obj.last_checked = obj.STATUS_REMOVED, obj.CHECK_NONE
                 obj.save()
-                MsgLog(LOG, request).info("Removed: {o}".format(o=obj))
-            else:
-                MsgLog(LOG, request).info("Already removed: {o}".format(o=obj))
 
         @classmethod
         def mbd_action(cls, request, queryset, action, **kwargs):
@@ -355,9 +345,26 @@ class StatusModel(Model):
             """
             for o in queryset:
                 try:
-                    getattr(cls, "mbd_" + action)(request, o, **kwargs)
+                    getattr(cls, "mbd_" + action)(o, **kwargs)
+                    msg = f"{action} succeeded: {o}"
+                    LOG.info(msg)
+                    if request is not None:
+                        django.contrib.messages.add_message(request, django.contrib.messages.INFO, msg)
                 except BaseException as e:
-                    mini_buildd.config.log_exception(MsgLog(LOG, request), "{a} failed: {o}".format(a=action, o=o), e)
+                    msg = f"{action} failed: {o}"
+                    mini_buildd.log_exception(LOG, msg, e)
+                    if request is not None:
+                        django.contrib.messages.add_message(request,
+                                                            django.contrib.messages.ERROR,
+                                                            django.utils.html.format_html("{}: {} (see <a href='{}daemon.log'>daemon.log</a>)",
+                                                                                          msg,
+                                                                                          str(mini_buildd.e2http(e)),
+                                                                                          mini_buildd.config.URIS["log"]["view"].uri))
+
+        @classmethod
+        def mbd_actions(cls, request, queryset, actions):
+            for action in actions:
+                cls.mbd_action(request, queryset, action)
 
         def mbd_action_prepare(self, request, queryset):
             self.mbd_action(request, queryset, "prepare")
@@ -375,77 +382,57 @@ class StatusModel(Model):
             self.mbd_action(request, queryset, "deactivate")
         mbd_action_deactivate.short_description = "Deactivate"
 
-        def mbd_action_remove(self, request, queryset):  # pylint: disable=inconsistent-return-statements
+        def mbd_action_remove(self, request, queryset):
             if request.POST.get("confirm"):
                 self.mbd_action(request, queryset, "remove")
-            else:
-                return django.template.response.TemplateResponse(
-                    request,
-                    "admin/confirm.html",
-                    {
-                        "title": ("Are you sure?"),
-                        "queryset": queryset,
-                        "action": "mbd_action_remove",
-                        "desc": """\
-Unpreparing means all the data associated by preparation will be
-removed from the system. Especially for repositories,
-this would mean losing all packages!
-""",
-                        "action_checkbox_name": django.contrib.admin.helpers.ACTION_CHECKBOX_NAME})
+                return None
+            return django.template.response.TemplateResponse(
+                request,
+                "admin/mini_buildd/mbd_action_remove_confirm.html",
+                {
+                    "title": ("Are you sure?"),
+                    "queryset": queryset,
+                    "action": "mbd_action_remove",
+                    "desc": ("Unpreparing means all the data associated by preparation will be\n"
+                             "removed from the system. Especially for repositories,\n"
+                             "this would mean losing all packages!\n"),
+                    "action_checkbox_name": django.contrib.admin.helpers.ACTION_CHECKBOX_NAME})
         mbd_action_remove.short_description = "Remove"
 
         def mbd_action_pc(self, request, queryset):
-            self.mbd_action(request, queryset, "prepare")
-            self.mbd_action(request, queryset, "check")
+            self.mbd_actions(request, queryset, ["prepare", "check"])
         mbd_action_pc.short_description = "PC"
 
         def mbd_action_pca(self, request, queryset):
-            self.mbd_action_pc(request, queryset)
-            self.mbd_action(request, queryset, "activate")
+            self.mbd_actions(request, queryset, ["prepare", "check", "activate"])
         mbd_action_pca.short_description = "PCA"
 
         @classmethod
-        def mbd_meta_pca_all(cls, msglog):
+        def mbd_pca_all(cls, request):
             """Run prepare, check, and activate for all objects of this model."""
-            cls.mbd_action(msglog.request, cls.mbd_model.objects.all(), "prepare")
-            cls.mbd_action(msglog.request, cls.mbd_model.objects.all(), "check")
-            cls.mbd_action(msglog.request, cls.mbd_model.objects.all(), "activate")
+            cls.mbd_actions(request, cls.mbd_model.objects.all(), ["prepare", "check", "activate"])
 
-        def colored_status(self, obj):  # pylint: disable=no-self-use
-            return django.utils.html.format_html(
-                '<div style="font-weight:bold;background-color:{bc};color:{fc};padding:2px 0px 2px 5px" title="{t}">{o}</div>',
-                bc=obj.STATUS_COLORS[obj.status].get("bg"),
-                fc=obj.STATUS_COLORS[obj.status].get("fg"),
-                t=obj.mbd_get_status_display(typ="string"),
-                o=obj.mbd_get_status_display(typ="char"))
+        @classmethod
+        def colored_status(cls, obj):
+            return django.utils.html.format_html(f'<span class="{mbd_cssmap("model_status", obj.get_status_display())} {"mbd-attention" if obj.mbd_needs_attention() else ""}" title="{obj.mbd_get_status_display(typ="string")}">{obj.mbd_get_status_display(typ="char")}</span>')
 
         actions = [mbd_action_prepare, mbd_action_check, mbd_action_pc, mbd_action_activate, mbd_action_pca, mbd_action_deactivate, mbd_action_remove]
         list_display = ["colored_status", "__str__"]
         list_display_links = ["__str__"]
 
-    @property
-    def days_until_recheck(self):
-        """
-        Field temporarily implemented as extra_option.
-
-        .. note:: Currently using 6 days as default value -- as it copes better with the current setup to restart
-                  mini-buildd weekly (via cron.weekly) to actually achieve automated checking.
-        """
-        return int(self.mbd_get_extra_option("Days-Until-Recheck", "6"))
-
-    def mbd_set_changed(self, request):
+    def mbd_set_changed(self):
         if self.mbd_is_active():
             self.status = self.STATUS_PREPARED
-            MsgLog(LOG, request).warning("Deactivated due to changes: {o}".format(o=self))
+            LOG.warning(f"Deactivated due to changes: {self}")
         self.last_checked = self.CHECK_CHANGED
-        MsgLog(LOG, request).warning("Marked as changed: {o}".format(o=self))
+        LOG.warning(f"Marked as changed: {self}")
 
     #
     # Action hooks helpers
     #
-    def _mbd_remove_and_prepare(self, request):
-        mini_buildd.models.base.StatusModel.Admin.mbd_remove(request, self)
-        mini_buildd.models.base.StatusModel.Admin.mbd_prepare(request, self)
+    def _mbd_remove_and_prepare(self):
+        mini_buildd.models.base.StatusModel.Admin.mbd_remove(self)
+        mini_buildd.models.base.StatusModel.Admin.mbd_prepare(self)
 
     #
     # Status abstractions and helpers
@@ -459,12 +446,12 @@ this would mean losing all packages!
     def mbd_is_checked(self):
         return self.last_checked > self._CHECK_MAX
 
-    def mbd_needs_check(self):
-        return not self.mbd_is_checked() or self.last_checked < (django.utils.timezone.now() - datetime.timedelta(days=self.days_until_recheck))
-
     def mbd_is_changed(self):
         return self.last_checked == self.CHECK_CHANGED
 
+    def mbd_needs_attention(self):
+        return self.last_checked == self.CHECK_REACTIVATE or self.mbd_is_changed()
+
     @classmethod
     def mbd_get_active(cls):
         return cls.objects.filter(status__gte=cls.STATUS_ACTIVE)
@@ -477,9 +464,17 @@ this would mean losing all packages!
     def mbd_get_prepared(cls):
         return cls.objects.filter(status__gte=cls.STATUS_PREPARED)
 
+    @classmethod
+    def mbd_get_needs_attention(cls):
+        return [o for o in cls.objects.all() if o.mbd_needs_attention()]
+
+    @classmethod
+    def mbd_attention(cls):
+        """Return human-readable list of objects needing attention (one by line)."""
+        return [f"{obj.__class__.__name__}: {obj} needs attention." for obj in cls.mbd_get_needs_attention()]
+
     def mbd_get_check_display(self, typ="string"):
         return {"string": self.last_checked.strftime("Checked on %Y-%m-%d %H:%M"), "char": "C"}[typ] if self.mbd_is_checked() else self.CHECK_STRINGS[self.last_checked][typ]
 
     def mbd_get_status_display(self, typ="string"):
-        return "{s} ({p})".format(s=self.get_status_display(),
-                                  p=self.mbd_get_check_display(typ))
+        return f"{self.get_status_display()} ({self.mbd_get_check_display(typ)})"
