@@ -11,6 +11,4 @@ class Subscription(mini_buildd.models.base.Model):
     distribution = django.db.models.CharField(max_length=100, blank=True)
 
     def __str__(self):
-        return "User '{u}' subscribes to '{p}' in '{d}'".format(u=self.subscriber,
-                                                                p=self.package if self.package else "any package",
-                                                                d=self.distribution if self.distribution else "any distribution")
+        return f"User '{self.subscriber}' subscribes to '{self.package if self.package else 'any package'}' in '{self.distribution if self.distribution else 'any distribution'}'"
