@@ -1,13 +1,7 @@
 import logging
 
-import django.conf
 import django.core.handlers.wsgi
 import django.core.management
-import django.contrib.messages.constants
-
-import mini_buildd.config
-import mini_buildd.models
-import mini_buildd.models.msglog
 
 LOG = logging.getLogger(__name__)
 
@@ -18,22 +12,21 @@ class WebApp(django.core.handlers.wsgi.WSGIHandler):
     def __init__(self):
         LOG.info("Generating web application...")
         super().__init__()
-        mini_buildd.models.import_all()
 
         LOG.info("Migrating database (migrate)...")
         try:
             django.core.management.call_command("migrate", interactive=False, run_syncdb=True, verbosity=0)
         except django.db.OperationalError as e:
-            LOG.warning("OperationalError on migrate ({}). Retrying with '--fake-initial'...".format(e))
+            LOG.warning(f"OperationalError on migrate ({e}). Retrying with '--fake-initial'...")
             django.core.management.call_command("migrate", interactive=False, run_syncdb=True, fake_initial=True, verbosity=0)
 
-        LOG.info("Clean up python-registration (cleanupregistration)...")
-        django.core.management.call_command("cleanupregistration", verbosity=0)
-
         # django 1.8 no longer per default runs this; Doing this once, manually
         LOG.info("Run django internal checks (check)...")
         django.core.management.call_command("check")
 
+        # For temporary debugging/check settings only
+        # LOG.debug(django.core.management.call_command("diffsettings", verbosity=0, output="unified"))
+
     @classmethod
     def set_admin_password(cls, password):
         """
@@ -71,5 +64,5 @@ class WebApp(django.core.handlers.wsgi.WSGIHandler):
 
     @classmethod
     def dumpdata(cls, app_path):
-        LOG.info("Dumping data for: {a}".format(a=app_path))
+        LOG.info(f"Dumping data for: {app_path}")
         django.core.management.call_command("dumpdata", app_path, indent=2, format="json")
